#include "MMA8452Q.h"

void MMA8452_init(int32 mask){
   int temp = 0;
   int temp2 = 0;
   int REG1_mask = 0;
   int REG2_mask = 0;
   int REG3_mask = 0;
   int REG4_mask = 0;
   int REG5_mask = 0;
   
   REG1_mask = mask;

   temp = mask >> 8;
   REG2_mask = temp & 0x07;
   temp2 = temp & 0x03;
   temp2 = temp2 << 3;
   REG2_mask |= temp2;
   temp2 = temp & 0x18;
   temp2 = temp2 << 3;
   REG2_mask |= temp2;

   temp = mask >> 13;
   REG3_mask = temp & 0x03;
   temp2 = temp & 0x1C;
   temp2 = temp2 << 1;
   REG3_mask |= temp2;
   temp2 = temp & 0x40;
   REG3_mask |= temp2;

   temp = mask >> 20;
   REG4_mask = temp & 0x01;
   temp2 = temp & 0x1E;
   temp2 = temp2 << 1;
   REG4_mask |= temp2;
   temp2 = temp & 0x20;
   temp2 = temp2 << 2;
   REG4_mask |= temp2;

   temp = mask >> 26;
   REG5_mask = temp & 0x01;
   temp2 = temp & 0x1E;
   temp2 = temp2 << 1;
   REG5_mask |= temp2;
   temp2 = temp & 0x20;
   temp2 = temp2 << 2;
   REG5_mask |= temp2;
   
   writeReg(SYSMOD, 0x01);
   writeReg(CTRL_REG1, REG1_mask);
   writeReg(CTRL_REG2, REG2_mask);
   writeReg(CTRL_REG3, REG3_mask);
   writeReg(CTRL_REG4, REG4_mask);
   writeReg(CTRL_REG5, REG5_mask);
   REG1_mask |= 0x01;                  
   writeReg(CTRL_REG1, REG1_mask);  
}

///////////////////////////////////////////////////////////////////////////////
//       You can write any data on any register using this function          //
///////////////////////////////////////////////////////////////////////////////
void writeReg(unsigned int reg, unsigned int data){
   i2c_start();
   i2c_write(DIR_ESCLAVO_W);
   i2c_write(reg);
   i2c_write(data);
   i2c_stop();
}

///////////////////////////////////////////////////////////////////////////////
//       You can read the content of any register using this function        //
///////////////////////////////////////////////////////////////////////////////
int readReg(unsigned int reg){
   int y;
   i2c_start();
   i2c_write(DIR_ESCLAVO_W);
   i2c_write(reg);
   i2c_start();
   i2c_write(DIR_ESCLAVO_R);
   y = i2c_read(0);
   i2c_stop();
   return y;
}

///////////////////////////////////////////////////////////////////////////////
//    is_connected_MMA8452Q();                                               //
///////////////////////////////////////////////////////////////////////////////
int MMA8452_is_connected(){
   int status = 0;
   if (readReg(who_am_I) == 0x2A){
      status = 1;
   }
   else{
      status = 0;
    }
    return status;
}
        
///////////////////////////////////////////////////////////////////////////////
void MMA8452_standby(){
   int temp = 0;
   temp = readReg(CTRL_REG1); 
   writeReg(CTRL_REG1, temp & 0xFE); 
}

void MMA8452_active(){
   int temp = 0;
   temp = readReg(CTRL_REG1);
   writeReg(CTRL_REG1, temp | 0x01);
}

void MMA8452_filter(int mask){
   MMA8452_standby();
   writeReg(HP_FILTER_CUTOFF, mask);
   MMA8452_active();
}

///////////////////////////////////////////////////////////////////////////////
//             Read and configure the X-axis, Y-axis and Z-axis              //                     
///////////////////////////////////////////////////////////////////////////////
void MMA8452_xyz_conf(int mask){
   MMA8452_standby();
   writeReg(XYZ_DATA_CFG,mask);
   MMA8452_active();
}

signed long int MMA8452_x_read(){
   signed long int x = 0;
   int xh = 0;
   int xl = 0;
   signed long int temp = 0;
   xh =  readReg(OUT_X_MSB);
   xl =  readReg(OUT_X_LSB);
   x =   make16(xh,xl);
   x =   x >> 4;
   if (x >= 2048){
      temp = x | 0xF000;
      temp = ~temp + 1;
      x = temp;
      x = ~x+1;
   }
   return x;
}

signed long int MMA8452_y_read(){
   signed long int y = 0;
   int yh = 0;
   int yl = 0;
   signed long int temp = 0;
   yh =  readReg(OUT_Y_MSB);
   yl =  readReg(OUT_Y_LSB);
   y =   make16(yh,yl);
   y =   y >> 4;
   if (y >= 2048){
      temp = y | 0xF000;
      temp = ~temp + 1;
      y = temp;
      y = ~y+1;
   }
   return y;
}

signed long int MMA8452_z_read(){
   signed long int z = 0;
   int zh = 0;
   int zl = 0;
   signed long int temp = 0;
   zh =  readReg(OUT_Z_MSB);
   zl =  readReg(OUT_Z_LSB);
   z =   make16(zh,zl);
   z =   z >> 4;
   if (z >= 2048){
      temp = z | 0xF000;
      temp = ~temp + 1;
      z = temp;
      z = ~z+1;
   }
   return z;
}

///////////////////////////////////////////////////////////////////////////////
//                            Landscape/Portrait                             //
///////////////////////////////////////////////////////////////////////////////
void MMA8452_enPortLand(){
   int   temp = 0;
   MMA8452_standby();
   temp = readReg(PL_CFG);
   writeReg(PL_CFG, temp | 0x40);
   writeReg(PL_COUNT, 0x05); 
   MMA8452_active();
}

int   MMA8452_newOrientation(){
   int temp = 0;
   temp = readReg(PL_STATUS);
   temp = temp >> 7;
   return temp;
}

int MMA8452_LaPo(){
   int temp = 0;
   temp = readReg(PL_STATUS);
   temp = temp >> 1;
   temp = temp & 0x03;
   return temp;
}

int   MMA8452_BaFro(){
   int temp = 0;
   temp = readReg(PL_STATUS);
   temp = temp & 0x01;
   return temp;
}

///////////////////////////////////////////////////////////////////////////////
//          Single/Double tap detection
///////////////////////////////////////////////////////////////////////////////
void MMA8452_enTapDet(int mask){
   int   temp = 0;
   MMA8452_Standby();
   temp = readReg(CTRL_REG1);
   temp &=  0xC7;
   temp |= DR_DEFAULT;
   writeReg(CTRL_REG1, temp); //ODR DEFAULT
   temp = readReg(CTRL_REG2);
   temp &= 0xE4;
   writeReg(CTRL_REG2, temp); //Normal Mode
   
   switch   (mask & 0x03){
      case  0:
         writeReg(PULSE_CFG, 0x55);
         break;
      case  1:
         writeReg(PULSE_CFG, 0x6A);
         break;
      case  2:
         writeReg(PULSE_CFG, 0x7F);
         break;
      default:
         writeReg(PULSE_CFG, 0x7F);
         break;
   }
   
   temp = X_G / 0.063;
   writeReg(PULSE_THSX, temp);
   temp = Y_G / 0.063;
   writeReg(PULSE_THSY, temp);
   temp = Z_G / 0.063;
   writeReg(PULSE_THSZ, temp);
   
   MMA8452_filter(LPF_DIS);
   
   writeReg(PULSE_TMLT, TMLT_COUNT);
   writeReg(PULSE_LTCY, LTCY_COUNT);
   writeReg(PULSE_WIND, WIND_COUNT);
   
   MMA8452_Active();
}

int MMA8452_getTap(){
   int temp = 0;
   int temp2 = 0;
   temp = readReg(PULSE_SRC) & 0x88;
   if (temp == 0x88){
      temp2 = 1;
   }
   else if (temp == 0x80){
      temp2 = 2;
   }
   else{
      temp2 = 0;
   }
   return temp2;
}
