///////////////////////////////////////////////////////////////////////////////
//                 Configura la ubicacin de los pines                       //
///////////////////////////////////////////////////////////////////////////////
#define  INT1_PIN       PIN_B0   //Interruption 1
#define  INT2_PIN       PIN_B1   //Interruption 2
#define  SCL_PIN        PIN_B2   //S Clock
#define  SDA_PIN        PIN_B3   //S Data
#define  DR_DEFAULT     DR_800   //Data Rate usado para las funciones embebidas
                                 //[800] default
                                 //DR_x : 800, 200, 100, 50, 12_5, 6_25 or 1_56
                                 
///////////////////////////////////////////////////////////////////////////////
//               Configurar si se usa la deteccin de pulsos                 //
///////////////////////////////////////////////////////////////////////////////
#define  X_G            3        
#define  Y_G            3
#define  Z_G            3
//El conteo para cada registro, 800 Hz ODR y modo normal (DEFAULT)
//Para ms informacin revisar la hoja de datos del MMA8452Q
#define  TMLT_COUNT     96       // [60ms] / [0.625ms] = 96
#define  LTCY_COUNT     160      // [200ms] / [1.25ms] = 160
#define  WIND_COUNT     160      // [200ms] / [1.25ms] = 160

///////////////////////////////////////////////////////////////////////////////
//                         CONFIGURACIN I2C                                 //
///////////////////////////////////////////////////////////////////////////////
#use     I2C(MASTER, SCL=SCL_PIN, SDA=SDA_PIN)
#define  DIR_ESCLAVO_W     0x3A  //Direccin esclava para escritura [8 bits]
#define  DIR_ESCLAVO_R     0x3B  //Direccin esclava para lectura [8 bits]


///////////////////////////////////////////////////////////////////////////////
//  Esta biblioteca permite al usuario escribir y leer los registros del     //
//  sensor MMA8452Q, as como utilizar las funciones embebidas del mismo.    //
//                                                                           //
//  Es importante configurar los pines de la librera para que se adecuen    //
//  a las conexiones de tu tarjeta. As como calcular algunas constantes     //
//  que dependen de la ODR seleccionada y el modo del sensor.                //
//                                                                           //
/////////////////////////  INICIALIZACION   ///////////////////////////////////
//                                                                           //
//  Antes de usar la biblitoeca, el usuario tiene que implementar la         //
//  siguiente funcin:                                                       //
//                                                                           //
//       void MMA8452_init(int32 mask) : Inicializa el sensor                //
//                configurando los registros de control.                     //
//       Revisar las constantes usadas para la funcin MMA8452_init()        //
//                                                                           //
/////////////////////////////  REGISTROS  /////////////////////////////////////
//                                                                           //
//  El usuario puede escribir o leer cualquier registro utilizando las       //
//  siguientes funciones:                                                    //
//                                                                           //
//       void writeReg(unsigned int reg, unsigned int data) : Escribe e      //
//                   "data" en el "reg", usando I2C.                         //
//                                                                           //
//       int readReg(unsigned int reg) : Lee el registro especificado,       //
//                   usando I2C                                              //
//                                                                           //
/////////////////////////////////// CONECTADO? ///////////////////////////////
//                                                                           //
//  El usuario puede revisar si el dispositivo est conectado con la         //
//  siguiente funcin:                                                       //
//                                                                           //
//       int MMA8452_is_connected() : Lee el registro who am I               //
//                   y compara la informacin recibida por I2C con la        //
//                   informacin del sensor                                  //
//               Regresa MMA8452_CONNECTED : Si est conectado               //
//               Regresa MMA8452_NOT_CONNECTED : Si no est conectado        //
//                                                                           //
/////////////////////////////// EJE-X EJE-Y EJE-Z  ////////////////////////////
//                                                                           //
//  Para leer y configurar los ejes del sensor
//                                                                           //
//       void MMA8452_xyz_conf(int mask) : Configura el rango de escala y    //
//                  el filtro pasa altas                                     //
//                Revisar las constantes usadas para MMA8452_xyz_conf();     //
//                                                                           //
//       signed long int MMA8452_x_read() : Regresa el valor del eje X       //
//                                                                           //
//       signed long int MMA8452_y_read() : Regresa el valor del eje Y       //
//                                                                           //
//       signed long int MMA8452_z_read() : Regresa el valor del eje Z       //
//                                                                           //
/////////////////////////////   ORIENTACION  //////////////////////////////////
//                                                                           //
//  Para usar la deteccin de orientacin (Portaretraro o Paisaje) tienes    //
//  que habilitarla utilizando la funcin MMA8452_enPortLand()               //
//                                                                           //
//       void MMA8452_enPortLand() : Habilita la deteccin de orientacin    //
//                                                                           //
//       int MMA8452_newOrientation() :                                      //
//          Regresa MMA8452_ORIEN_CHANGED : Si la orientacin cambi         //
//          Regresa MMA8452_ORIEN_NOT_CHANGED : Si la orientacin no cambi  //
//                                                                           //
//       int MMA8452_LaPo() :                                                //
//                Return MMA8452_PORTRAIT_UP : Portaretrato arriba           //
//                Return MMA8452_PORTRAIT_DOWN : Portaretrato abajo          //
//                Return MMA8452_LANDSCAPE_RIGHT : Paisaje derecha           //
//                Return MMA8452_LANDSCAPE_LEFT : Paisaje izquierda          //
//                                                                           //
//       int MMA8452_BaFro() :                                               //
//                Return MMA8452_FRONT_FACE : Boca arriba                    //
//                Return MMA8452_BACK_FACE : Boca abajo                      //
//                                                                           //
//////////////////////////   DETECCIN DE PULSO  //////////////////////////////
//                                                                           //
// Para usar la deteccin de pulsos, el usuario tiene que habilitar esta
// funcion embebida utilizando la funcin MMA8452_enTapDet()
//                                                                           //
//       void MMA8452_enTapDet(int mask) : El usuario tiene que seleccionar  //
//                   si se usar deteccin simple y/o doble.                 //
//                Revisar las constantes usadas para for MMA8452_enTapDet()  //
//                                                                           //
//       int MMA8452_getTap() :                                              //
//                Regresa MMA8452_NO_PULSE : No detect pulso                //
//                Regresa MMA8452_DOUBLE_PULSE : Detect pulso doble         //
//                Regresa MMA8452_SINGLE_PULSE : Detect pulso simple        //
///////////////////////////////////////////////////////////////////////////////

#define  MMA8452_CONNECTED          1
#define  MMA8452_NOT_CONNECTED      0
#define  MMA8452_ORIEN_CHANGED      1
#define  MMA8452_ORIEN_NOT_CHANGED  0
#define  MMA8452_PORTRAIT_UP        0
#define  MMA8452_PORTRAIT_DOWN      1
#define  MMA8452_LANDSCAPE_RIGHT    2  
#define  MMA8452_LANDSCAPE_LEFT     3
#define  MMA8452_FRONT_FACE         0
#define  MMA8452_BACK_FACE          1
#define  MMA8452_NO_PULSE           0
#define  MMA8452_DOUBLE_PULSE       1
#define  MMA8452_SINGLE_PULSE       2

///////////////////////////////////////////////////////////////////////////////
//                         Direcciones de los registros                      //
///////////////////////////////////////////////////////////////////////////////
#define  OUT_X_MSB         0x01
#define  OUT_X_LSB         0x02
#define  OUT_Y_MSB         0x03
#define  OUT_Y_LSB         0x04
#define  OUT_Z_MSB         0x05
#define  OUT_Z_LSB         0x06
#define  SYSMOD            0x0B
#define  INT_SOURCE        0x0C
#define  WHO_AM_I          0x0D
#define  XYZ_DATA_CFG      0x0E
#define  HP_FILTER_CUTOFF  0x0F
#define  PL_STATUS         0x10
#define  PL_CFG            0x11
#define  PL_COUNT          0x12
#define  PL_BF_ZCOMP       0x13
#define  P_L_THS_REG       0x14
#define  FF_MT_CFG         0x15
#define  FF_MT_SRC         0x16
#define  FF_MT_THS         0x17
#define  FF_MT_COUNT       0x18
#define  PULSE_CFG         0x21
#define  PULSE_SRC         0x22
#define  PULSE_THSX        0x23
#define  PULSE_THSY        0x24
#define  PULSE_THSZ        0x25
#define  PULSE_TMLT        0x26
#define  PULSE_LTCY        0x27
#define  PULSE_WIND        0x28
#define  CTRL_REG1         0x2A
#define  CTRL_REG2         0x2B
#define  CTRL_REG3         0x2C
#define  CTRL_REG4         0x2D
#define  CTRL_REG5         0x2E


///////////////////////////////////////////////////////////////////////////////
//                    Constantes usadas para MMA8452_init()                  //
///////////////////////////////////////////////////////////////////////////////
//       Valores por defecto:                                                //
//          ASLP Rate: 50Hz                                                  //
//          Data rate: 800Hz                                                 //
//          Noise: Normal                                                    //
//          Fast Read: Not enable                                            //
//          Normal mode                                                      //
//          No Autosleep                                                     //
//          No software reset                                                //
//          No selftest enabled                                              //
//          Push Pull                                                        //
//          IPOL low                                                         //
//          No interrupts                                                    //
///////////////////////////////////////////////////////////////////////////////
//Frecuencia de muestreo de Auto wake up cuando el dispositivo est dormido
#define  ASLP_RATE_50      0x00000000     // 50Hz
#define  ASLP_RATE_12_5    0x00000040     // 12.5 Hz
#define  ASLP_RATE_6_25    0x00000080     // 6.25 Hz
#define  ASLP_RATE_1_56    0x000000C0     // 1.56 Hz
//Seleccion de data rate
#define  DR_800            0x00000000     // 800 Hz
#define  DR_400            0x00000008     // 400 Hz
#define  DR_200            0x00000010     // 200 Hz
#define  DR_100            0x00000018     // 100 Hz
#define  DR_50             0x00000020     // 50 Hz
#define  DR_12_5           0x00000028     // 12.5 Hz
#define  DR_6_25           0x00000030     // 6.25 Hz
#define  DR_1_56           0x00000038     // 1.56 Hz

#define  NOISE_NORMAL      0x00000000     // Ruido normal
#define  NOISE_RED         0x00000004     // Ruido reducido

#define  NO_FAST_READ      0x00000000     // Modo normal
#define  FAST_READ         0x00000002     // Modo de lectura rpida

//Power mode
#define  NORMAL_MODE       0x00000000     // Normal
#define  LOW_NOISEPOWER    0x00000100     // Bajo ruido baja energa
#define  HIGH_RES          0x00000200     // Alta resolucin
#define  LOW_POWER         0x00000300     // Baja potencia

#define  NO_AUTOSLEEP      0x00000000     // Autosleep no habilitado
#define  AUTOSLEEP         0x00000400     // Autosleep habilitado

#define  SW_RST            0x00000800     // Reset por software habilitado
#define  NO_SWR            0x00000000     // Reset por software no habilitado

#define  SELFTEST          0x00001000     // Self test habilitado
#define  NO_SELFTEST       0x00000000     // Self test deshabilitado

#define  PUSH_PULL         0x00000000     // Push pull
#define  OPEN_DRAIN        0x00002000     // Open drain

//Polaridad en la seal de interrupcin
#define  IPOL_LOW          0x00000000     // Activa bajo
#define  IPOL_HIGH         0x00004000     // Activa alto

//Funciones que pueden despertar al sistema o no en modo DORMIDO
#define  WAKE_FFMT         0x00008000     // Cada libre / Movimiento
#define  WAKE_FFMT_OFF     0x00000000     // Cada libre / Movimiento apagado
#define  WAKE_PULSE        0x00010000     // Pulso
#define  WAKE_PULSE_OFF    0x00000000     // Pulso apagado
#define  WAKE_LNDPRT       0x00020000     // Paisaje / Portaretrato
#define  WAKE_LNDPRT_OFF   0x00000000     // Paisaje / Portaretrato apagado
#define  WAKE_TRANS        0x00080000     // Transient
#define  WAKE_TRANS_OFF    0x00000000     // Transient apagado

#define  INT1_DATAREADY    0x04100000     // Interrupcin de DATOS LISTOS conectada a INT1 
#define  INT2_DATAREADY    0x00100000     // Interrupcin de DATOS LISTOS conectada a INT2 
#define  INT1_FFMT         0x08200000     // Interrupcin de Cada libre / Movimiento conectada a INT1
#define  INT2_FFMT         0x00200000     // Interrupcin de Cada libre / Movimiento conectada a INT2
#define  INT1_PULSE        0x10400000     // Interrupcin de pulso conectada a INT1
#define  INT2_PULSE        0x00400000     // Interrupcin de pulso conectada a INT1
#define  INT1_LNDPRT       0x20800000     // Interrupcin de Paisaje / Portaretrato conectada a INT1
#define  INT2_LNDPRT       0x00800000     // Interrupcin de Paisaje / Portaretrato conectada a INT2
#define  INT1_TRANS        0x41000000     // Interrupcin de Transient conectada a INT1
#define  INT2_TRANS        0x01000000     // Interrupcin de Transient conectada a INT2
#define  INT1_ASLP         0x82000000     // Interrupcin Auto dormir/despertar conectada a INT1
#define  INT2_ASLP         0x02000000     // Interrupcin Auto dormir/despertar conectada a INT2

///////////////////////////////////////////////////////////////////////////////
//                  Constantes usadas para MMA8452_enTapDet()                //
///////////////////////////////////////////////////////////////////////////////
//Deteccin de pulso
#define  SINGLE_TAP        0b00000000     //Deteccin sencilla activada
#define  DOUBLE_TAP        0b00000001     //Deteccin doble activada
#define  SINGLE_DOUBLE     0b00000010     //Deteccin sencilla y doble activada

///////////////////////////////////////////////////////////////////////////////
//                  Constantes usadas para MMA8452_xyz_conf()                //
///////////////////////////////////////////////////////////////////////////////
#define  HIGH_PASS_ON      0b00010000     //Activar pasa altas en datos de salida
#define  HIGH_PASS_OFF     0b00000000     //Filtrar los datos de salida
//Rangos de valores de escala
#define  SET_2G            0b00000000     //2g
#define  SET_4G            0b00000001     //4g
#define  SET_8G            0b00000010     //8g

///////////////////////////////////////////////////////////////////////////////
//                  Constantes usadas para MMA8452_filter()                  //
///////////////////////////////////////////////////////////////////////////////
//Seleccin de frecuencia de corte de filtro pasa altas (para ms informacin//
//revisar el registro 0x0F: HP_FILTER_CUTOFF en la hoja de datos             //
#define  SEL0              0b00000000     // Seleccion 0
#define  SEL1              0b00000001     // Seleccion 1
#define  SEL2              0b00000010     // Seleccion 2
#define  SEL3              0b00000011     // Seleccion 3
#define  HPF_EN            0b00000000     // Filtro pasa altas activo
#define  HPF_BYP           0b00100000     // Filtro pasa altas desactivado
#define  LPF_EN            0b00010000     // Filtro pasa bajas activo
#define  LPF_DIS           0b00000000     // Filtro pasa bajas desactivado

///////////////////////////////////////////////////////////////////////////////
//                            Prototipos de funciones                        //
///////////////////////////////////////////////////////////////////////////////
//    Escribir o leer del registro
void              writeReg(unsigned int reg, unsigned int data);
int               readReg(unsigned int reg);
//    Configuracin e inicializacin
void              MMA8452_init();
void              MMA8452_active();
void              MMA8452_standby();
int               MMA8452_is_connected();
void              MMA8452_xyz_conf(int mask);
void              MMA8452_filter(int mask);
//    Lectura de los ejes x,y y z
signed long int   MMA8452_x_read();
signed long int   MMA8452_y_read();
signed long int   MMA8452_z_read();
//    Orientacin Portaretrato o Paisaje
void              MMA8452_enPortLand();
int               MMA8452_newOrientation();
int               MMA8452_LaPo();
int               MMA8452_BaFro();
//    Deteccin de pulsos sencillos y / o dobles
void              MMA8452_enTapDet(int mask);
int               MMA8452_getTap();
